#!/usr/bin/perl

# this script is used to combine two .bedGraph files
# and generate a .wig file

use List::Util qw(first max maxstr min minstr reduce shuffle sum);

if ($#ARGV ne 4) {
  print "command line example: perl combine_two_bedGraph.pl raw_read_length chromosome_size_file file1.bedGraph file2.bedGraph merge.wig\n";
  exit;
}


$flag_norm = 0;
$raw_read_length = $ARGV[0];
$chr_size = $ARGV[1];
$infile1 = $ARGV[2];
$infile2 = $ARGV[3];
$outfile = $ARGV[4];


open (in, "<$chr_size");
while ($line=<in>) {
  chomp $line;
  @data = split /[\s+\t+]/, $line;
  $chr_size{$data[0]} = $data[1];
  push (@chrs, $data[0]);
}
close in;


open (out, ">temp");
#print out "track type=wiggle_0\n";

foreach $chr (@chrs) {
  print "$chr\n";
  print out "variableStep chrom=$chr\n";

  %value1 = ();
  %value2 = ();
  open (in, "<$infile1");
  while ($line=<in>) {

    $count ++;
    if ($count%1000000 eq 0) {
      print "$count\n";
    }

    if (!($line=~/^track/)) {
      chomp $line;
      @data = split /[\s+\t+]/, $line;
      if ($chr eq $data[0]) {
	$start = $data[1];
	$end = $data[2];
	
	for ($i=$start; $i<$end; $i++) {
	  $value1{$i} = $data[3];
	}
      }
    }
  }
  close in;
  
  open (in, "<$infile2");
  while ($line=<in>) {

    $count ++;
    if ($count%1000000 eq 0) {
      print "$count\n";
    }

    if (!($line=~/^track/)) {
      chomp $line;
      @data = split /[\s+\t+]/, $line;
      if ($chr eq $data[0]) {
	$start = $data[1];
	$end = $data[2];
	
	for ($i=$start; $i<$end; $i++) {
	  $value2{$i} = $data[3];
	}
      }
    }
  }
  close in;
  
  for ($i=1; $i<$chr_size{$chr}; $i++) {
    if ($value1{$i} and $value2{$i}) {
      $pvalue = min($value1{$i}, $value2{$i});
      $sum = $sum + $pvalue;
      print out "$i\t$pvalue\n";
    }
  }
}

close out;

print "Total number of mappable reads = $sum/$raw_read_length\n";

if ($flag_norm) {

  open (in, "<temp");
  open (out, ">$outfile");
  while ($line=<in>) {
    if ($line=~/^track/) {
      print out $line;
    }
    elsif ($line=~/^variable/) {
      print out $line;
    }
    else {
      chomp $line;
      @data = split /[\t+\s+]/, $line;
      $pvalue = $data[1]*$raw_read_length*10000000/$sum;
      print out "$data[0]\t$pvalue\n";
    }
  }
  close in;
  close out;
  
  system "rm temp";
}
else {
  system "mv temp $outfile";
}
