#!/usr/bin/env python

import sys
import os.path
import re
from optparse import OptionParser
from collections import defaultdict

def options():
    usage = '''usage: %prog input1.sam input2.sam ... output.wig
                convert one .sam file to .wig file
                or aggregate multiple .sam file to .wig file
            '''
    parser = OptionParser(usage=usage)
    (options, args) = parser.parse_args()
    if len(args) < 2:
        parser.error("need input1.sam input2.sam ... output.wig!")
    return args

def loadsam(filenames):
    starts=defaultdict(int)
    ends=defaultdict(int)
    unaligned, aligned, spliced = 0,0,0 #bad and good alignment
    #global unaligned
    #global aligned
    for filename in filenames:
        if filename[-3:]==".gz":
            import gzip
            f=gzip.open(filename)
        else: f=open(filename, 'r')
        i=0
        for line in f:
            i+=1
            if i%1000000==0: print >> sys.stderr, i
            if line[0]=='@' or line[0]=='#': continue
            parts=line.strip().split('\t')
            (flag, chro, start)=parts[1:4]
            flagbin=bin(int(flag))
            unmapped='0' # '0' is mapped , 1 is unmapped
            try: unmapped =flagbin[-3]
            except: pass

            cigar = parts[5]
            if unmapped == '1' or chro=='*':
                unaligned+=1
                continue
            aligned+=1
            start= long(start)
            N=[]
            if cigar.find('N') == -1: # no splicing
                end = start + len(parts[9])
            else: #splicing
                N=re.findall(r'[0-9]+', cigar) #numbers
                end = start + int(N[0])
            try:
                starts[chro][start]+=1
                ends[chro][end]+=1
            except:
                starts[chro]=defaultdict(int)
                ends[chro]=defaultdict(int)
                starts[chro][start]=1
                ends[chro][end]=1

            if len(N)==3: #spliced alignment
                start2 = start + int(N[0])+ int(N[1])
                end2 = start2 + int(N[2])
                starts[chro][start2]+=1
                ends[chro][end2]+=1
                
        f.close()
    return starts, ends, unaligned, aligned

def printwig(outputfile, starts, ends):
    allkeys={}
    f=open(outputfile, 'w')
#    print >>f, "track type=bedGraph name="+os.path.basename(outputfile)
    for chro in starts.keys():
        allkeys[chro]=list(set(starts[chro].keys())|set(ends[chro].keys()))
        allkeys[chro].sort()
    
    for chro in allkeys.keys():
        counter=0
        prevkey=0
        allkey=allkeys[chro]
        start=starts[chro]
        end=ends[chro]
        for i in range(len(allkey)):
            key=allkey[i]
            #print chro, prevkey, key, counter
            if counter > 0:
		pcounter = round(float(counter)/float(aligned)*10000000, 2)
                print >> f, chro, prevkey, key, pcounter
            prevkey=key
            if start.has_key(key): counter+=start[key]
            if end.has_key(key): counter-=end[key]
    f.close()

if __name__ == '__main__':
    args= options()
    outputfile = args[-1]
    inputfiles = args[0:-1]
    starts, ends, unaligned, aligned = loadsam(inputfiles)
    printwig(outputfile, starts, ends)
    print 'sample', inputfiles
    print 'unaligned reads', unaligned
    print 'aligned reads', aligned
    try: print 'aligned percentage', round(float(aligned)/(aligned+unaligned),3)*100,'%'
    except: pass
